import inspect


def is_family_type(class1, class2) -> bool:
    """检查两个类型是否同族, 支持函数, 类, 若为函数则必须相等

    Args:
        class1: 类型1
        class2: 类型2
    """
    if inspect.isfunction(class1) or inspect.isfunction(class2):
        return class1 == class2
    return issubclass(class1, class2) or issubclass(class2, class1)



import inspect
import queue
from typing import Any, Type, Callable, Union

from dragons96_tools.ioc.beans.proxy import *
from dragons96_tools.ioc.utils import class_utils
from .base import BeanFactory
from .initializing_bean import InitializingBean

from dragons96_tools.ioc.utils.name_utils import to_camel_case


class _Param:
    name: str
    cls: Any
    primary: bool

    def __init__(self, name: str = None, cls: Any = None, primary: bool = False):
        self.name = name
        self.cls = cls
        self.primary = primary


class SimpleBeanFactory(BeanFactory):
    """简单bean工厂"""

    def __init__(self):
        self._name_bean = {}
        self._type_bean = {}
        self._init_queue = queue.Queue()
        self._dependency_map = {}
        self._register_class_object_bean(name=self.__class__.__name__, obj=self)

    def get_bean(self, name: str = None, required_type: Union[Type, Callable] = None) -> Any:
        if not name and not required_type:
            raise ValueError('bean名称和类型不能都为空')
        if name:
            return self._resolve_name_bean(name, required_type)
        return self._resolve_type_bean(required_type)

    def _resolve_bean_from_beans(self, beans):
        if len(beans) >= 2:
            for bean in beans:
                # 有primary返回primary
                if bean.primary():
                    return bean
            if len(beans) >= 2:
                raise RuntimeError('获取的bean数量不能超过1个, 请增加名称/类型获取bean或给某个bean配置primary属性')
        if beans:
            return beans[0]
        return None

    def _resolve_name_bean(self, name: str, required_type: Union[Type, Callable] = None):
        beans = self._name_bean.get(name)
        if not beans:
            return None
        # 特定类型
        if required_type:
            # 优先取相同类型的
            bean = self._resolve_bean_from_beans([bean for bean in beans if bean.ioc_type() == required_type])
            if bean:
                return bean
            # 同类型的未找到再找父类
            bean = self._resolve_bean_from_beans(
                [bean for bean in beans if class_utils.is_family_type(bean.ioc_type(), required_type)])
            if bean:
                return bean
        return self._resolve_bean_from_beans(beans)

    def _resolve_type_bean(self, required_type=None):
        beans = self._type_bean.get(required_type)
        if not beans:
            beans = []
            for _type in self._type_bean.keys():
                if class_utils.is_family_type(_type, required_type):
                    beans = [*beans, *self._type_bean[_type]]
        return self._resolve_bean_from_beans(beans)

    def _register_function_bean(self, name: str, func, primary: bool = False) -> Any:
        name = name or to_camel_case(func.__name__, upper_case=False)
        proxy = ClassBeanProxy(name=name, obj=func(), primary=primary)
        self._add_bean(proxy.ioc_name(), proxy.ioc_type(), proxy)
        return proxy

    def _register_class_object_bean(self, name, obj, primary: bool = False) -> Any:
        name = name or to_camel_case(obj.__class__.__name__, upper_case=False)
        proxy = ClassBeanProxy(name=name, obj=obj, primary=primary)
        self._add_bean(proxy.ioc_name(), proxy.ioc_type(), proxy)
        return proxy

    def _register_class_bean(self, name: str, cls, primary: bool = False) -> Any:
        return self._register_class_object_bean(name, cls(), primary=primary)

    def register_bean(self, name: str, cls, primary: bool = False, lazy=True) -> Any:
        # 注册bean懒加载, 在init方法执行bean创建逻辑
        if lazy:
            self._init_queue.put(_Param(name, cls, primary))
        else:
            self._register_bean(name, cls, primary)

    def _register_bean(self, name: str, cls, primary: bool = False) -> Any:
        if inspect.isfunction(cls):
            return self._register_function_bean(name, cls, primary=primary)
        if inspect.isclass(cls):
            return self._register_class_bean(name, cls, primary=primary)
        # 否则直接视为对象注册
        return self._register_class_object_bean(name, cls, primary=primary)

    def init(self):
        """初始化容器, 在bean都注册进工厂队列后执行, 同时解决多个容器间的依赖问题"""
        # 创建容器
        self._create_beans()
        # 初始化
        self._do_beans_init()

    def _do_beans_init(self):
        # 对所有容器执行__post_construct__, InitializingBean.after_properties_set 初始化方法
        for beans in self._name_bean.values():
            for bean in beans:
                # 容器原始类型是类才能执行
                if inspect.isclass(bean.ioc_type()):
                    if hasattr(bean.ioc_type(), '__post_construct__'):
                        bean.__post_construct__()
                    if issubclass(bean.ioc_type(), InitializingBean):
                        bean.after_properties_set()

    def _create_beans(self):
        while True:
            try:
                param: _Param = self._init_queue.get(block=False)
                # 没有依赖则可以直接注册bean
                if not self._has_dependency(param):
                    self._register_bean(param.name, param.cls, param.primary)
                    continue
                # todo: 依赖处理策略
            except queue.Empty:
                break

    def _has_dependency(self, param: _Param) -> bool:
        """判断要创建的bean是否存在依赖
        """
        # todo: 逻辑待实现
        return False

    def _add_bean(self, name: str, _type, bean: Any):
        self._add_name_bean(name, bean)
        self._add_type_bean(_type, bean)

    def _add_name_bean(self, name: str, bean: Any):
        beans = self._name_bean.get(name) or []
        beans.append(bean)
        self._name_bean[name] = beans

    def _add_type_bean(self, _type, bean: Any):
        beans = self._type_bean.get(_type) or []
        beans.append(bean)
        self._type_bean[_type] = beans



import importlib
import os


def get_all_py_modules(package_or_module_name: str):
    """获取包下的所有py模块

    Args:
        package_or_module_name: 指定包名或模块
    """
    package_or_module = importlib.import_module(package_or_module_name)
    py_modules = []
    if hasattr(package_or_module, '__path__'):
        package_path = package_or_module.__path__[0]
        for root, _, files in os.walk(package_path):
            for file in files:
                if file.endswith(".py") and file != "__init__.py":
                    module_path = os.path.relpath(os.path.join(root, file), package_path)
                    module_name = os.path.splitext(module_path)[0].replace(os.path.sep, '.')
                    py_modules.append(f"{package_or_module_name}.{module_name}")
        return py_modules
    # 否则本身就是一个模块
    py_modules.append(package_or_module_name)
    return py_modules



import importlib
from dragons96_tools.ioc.utils import reflect_utils
from dragons96_tools.ioc.base import new_bean_factory
from dragons96_tools.ioc.beans.factory import BeanFactory
from dragons96_tools.ioc.config import load_config
from typing import List, Union
from threading import RLock


class _Properties:
    running: int = 0
    lock = RLock()


def run(scan_package_names: Union[List[str], str], config_dir: str = None, factory: BeanFactory = None,
        exclude_modules: List[str] = None):
    """启动pyspring, 一个进程中重复启动仅第一次生效

    Args:
        scan_package_names: 需要扫描的包, 支持多个包, 示例: xxx.xxx; xxx
        config_dir: 配置目录, 若不为空则加将config_dir目录下的 application.yml, application-{env}.yml 文件(如果存在) 加载到系统配置中
        同时支持@Value, @Configuration装饰器通过IOC注入是属性和bean
        factory: bean工厂, 可自己实现, 默认走SimpleBeanFactory
        exclude_modules: 需要过滤的模块列表
    """
    # 启动加锁
    with _Properties.lock:
        if _Properties.running:
            return
        # 启动, 设置运行状态
        _Properties.running = 1
        if not scan_package_names:
            raise ValueError('包名不能为空')
        if isinstance(scan_package_names, str):
            scan_package_names = [scan_package_names]
        # 初始化工厂
        bean_factory = new_bean_factory(factory=factory)
        # 有配置则初始化配置
        if config_dir:
            from dragons96_tools.ioc.environment import Environment
            load_config(config_dir=config_dir)
            # 注册environment
            bean_factory.register_bean(name='environment', cls=Environment, primary=True, lazy=False)
        for scan_package_name in scan_package_names:
            # 加载包下所有某个和bean, 并注入到工厂
            modules = reflect_utils.get_all_py_modules(scan_package_name)
            for module in modules:
                if exclude_modules and module in exclude_modules:
                    continue
                importlib.import_module(module)
        # 初始化创建bean实例
        bean_factory.init()




from typing import Union, List

from .base import DataFileLoader, DataFileExtractor


class EvalDataFileLoader(DataFileLoader):

    def _convert_config_data_to_python(self,
                                       config_data: Union[str, bytes],
                                       **kwargs) -> Union[dict, tuple, list]:
        return eval(config_data)


class EvalDataFileExtractor(DataFileExtractor):

    def _convert_data_to_str(self, data: Union[dict, List[dict]], **kwargs) -> str:
        raise NotImplementedError('不支持当前类型文件提取')




from enum import Enum
from typing import Union, Type, Optional, List

from pydantic import BaseModel

from .base import DataFileLoader, DataFileExtractor
from .csv import CsvDataFileLoader, CsvDataFileExtractor
from .yaml import YamlDataFileLoader, YamlDataFileExtractor
from .json import JsonDataFileLoader, JsonDataFileExtractor
from .ini import IniDataFileLoader, IniDataFileExtractor
from .eval import EvalDataFileLoader, EvalDataFileExtractor
from .properties import PropertiesDataFileLoader, PropertiesDataFileExtractor
from .xml import XmlDataFileLoader, XmlDataFileExtractor
from .excel import ExcelDataFileLoader, ExcelDataFileExtractor

_csv_data_file_loader = CsvDataFileLoader()
_csv_data_file_extractor = CsvDataFileExtractor()
_yaml_data_file_loader = YamlDataFileLoader()
_yaml_data_file_extractor = YamlDataFileExtractor()
_json_data_file_loader = JsonDataFileLoader()
_json_data_file_extractor = JsonDataFileExtractor()
_ini_data_file_loader = IniDataFileLoader()
_ini_data_file_extractor = IniDataFileExtractor()
_properties_data_file_loader = PropertiesDataFileLoader()
_properties_data_file_extractor = PropertiesDataFileExtractor()
_eval_data_file_loader = EvalDataFileLoader()
_eval_data_file_extractor = EvalDataFileExtractor()
_xml_data_file_loader = XmlDataFileLoader()
_xml_data_file_extractor = XmlDataFileExtractor()
_excel_data_file_loader = ExcelDataFileLoader()
_excel_data_file_extractor = ExcelDataFileExtractor()


class DataType(Enum):
    """ 数据类型 """
    JSON = (('json',),
            _json_data_file_loader.load, _json_data_file_loader.load_file,
            _json_data_file_extractor.extract, _json_data_file_extractor.extract_file)
    YAML = (('yaml', 'yml'),
            _yaml_data_file_loader.load, _yaml_data_file_loader.load_file,
            _yaml_data_file_extractor.extract, _yaml_data_file_extractor.extract_file)
    INI = (('ini', 'toml', 'cfg'),
           _ini_data_file_loader.load, _ini_data_file_loader.load_file,
           _ini_data_file_extractor.extract, _ini_data_file_extractor.extract_file)
    PROPERTIES = (('properties',),
                  _properties_data_file_loader.load, _properties_data_file_loader.load_file,
                  _properties_data_file_extractor.extract, _properties_data_file_extractor.extract_file)
    CSV = (('csv', ),
           _csv_data_file_loader.load, _csv_data_file_loader.load_file,
           _csv_data_file_extractor.extract, _csv_data_file_extractor.extract_file)
    EVAL = (('eval',),
            _eval_data_file_loader.load, _eval_data_file_loader.load_file,
            _eval_data_file_extractor.extract, _eval_data_file_extractor.extract_file)
    XML = (('xml', 'html'),
           _xml_data_file_loader.load, _xml_data_file_loader.load_file,
           _xml_data_file_extractor.extract, _xml_data_file_extractor.extract_file)
    EXCEL = (('xls', 'xlsx'),
             _excel_data_file_loader.load, _excel_data_file_loader.load_file,
             _excel_data_file_extractor.extract, _excel_data_file_extractor.extract_file)


def get_file_data_type(file_path: str) -> Optional[DataType]:
    """获取文件的数据类型枚举, 若识别不了则返回None

    Args:
        file_path: 文件路径
    """
    file_type = file_path.split('.')[-1]
    file_data_type = None
    for data_type in DataType:
        if file_type in data_type.value[0]:
            file_data_type = data_type
            break
    return file_data_type


def _parse_data_type(data_type: Union[DataType, str]):
    if isinstance(data_type, DataType):
        return data_type
    if isinstance(data_type, str):
        data_type = data_type.lower()
        for item in DataType:
            if data_type in item.value[0]:
                return item
    raise ValueError('不支持的数据类型: {}'.format(data_type))


class AutoDataFileLoader(DataFileLoader):
    """
    动态检测数据/文件类型加载器
    """

    def load(self,
             config_data: Union[str, bytes],
             modelclass: Union[Type[dict], Type[BaseModel]] = dict,
             data_type: Union[DataType, str, None] = None,
             header=False,
             **kwargs) -> Union[dict, list, BaseModel]:
        """加载配置数据到python或pydantic模型
        Args:
            config_data: 数据字符串
            modelclass: 模型类型
            data_type: 数据类型, None 表示自动解析 (若知道数据类型推荐传入指定数据类型, 防止自动推断类型错误), 传入支持传入DataType枚举以及类型字符串如: json, xml, yaml, yml, properties等DataType支持的解析的类型
            header: 第一行是否为表头, 仅支持CSV传参
        """
        if data_type:
            data_type = self._parse_data_type(data_type)
            return data_type.value[1](config_data,
                                      modelclass=modelclass,
                                      header=header,
                                      use_header_as_key=header)

        for data_type in DataType:
            try:
                return data_type.value[1](config_data,
                                          modelclass=modelclass,
                                          header=header,
                                          use_header_as_key=header)
            except Exception:
                pass
        raise ValueError('无法解析的数据: ' + config_data)

    def load_file(self,
                  file_path: str,
                  encoding: str = 'utf-8',
                  modelclass: Union[Type[dict], Type[BaseModel]] = dict,
                  **kwargs) -> Union[dict, list, BaseModel]:
        """加载文件数据到python对象或pydantic模型

        Args:
            file_path: 文件路径
            encoding: 文件编码类型
            modelclass: 模型类型
        """
        data_type = get_file_data_type(file_path)
        if data_type:
            return data_type.value[2](file_path,
                                      encoding=encoding,
                                      modelclass=modelclass,
                                      **kwargs)

        for data_type in DataType:
            try:
                return data_type.value[2](file_path,
                                          encoding=encoding,
                                          modelclass=modelclass,
                                          **kwargs)
            except Exception:
                pass
        raise ValueError('无法解析的文件: ' + file_path)

    def _parse_data_type(self, data_type: Union[DataType, str]):
        return _parse_data_type(data_type)


DynamicDataFileLoader = AutoDataFileLoader


class DynamicDataFileExtractor(DataFileExtractor):
    """
    动态检测数据特定文件数据类型抽取器
    """

    def extract(self, data: Union[dict, List[dict], BaseModel, List[BaseModel]],
                data_type: Union[DataType, str] = DataType.JSON,
                header=True,
                **kwargs) -> str:
        """数据抽取为特定文件格式的字符串数据

        Args:
            data: 数据
            data_type: 数据类型, 支持DataType类型与字符串类型, 字符串类型传入: json, yaml, yml, csv, properties等DataType中支持的名称, 不支持的名称会抛出ValueError
            header: 第一行是否为表头, 仅支持CSV传参
        """
        data_type = self._parse_data_type(data_type)
        return data_type.value[3](data, header=header, **kwargs)

    def extract_file(self,
                     file_path: str,
                     data: Union[dict, List[dict], BaseModel, List[BaseModel]],
                     encoding: str = 'utf-8',
                     header=True,
                     **kwargs) -> str:
        """数据抽取为特定文件格式的文件字符串并保存至文件

        Args:
            file_path: 文件路径
            data: 数据
            encoding: 文件编码类型
            header: 第一行是否为表头, 仅支持CSV传参
        """
        data_type = get_file_data_type(file_path)
        return data_type.value[4](file_path, data, encoding=encoding, header=header, **kwargs)

    def _parse_data_type(self, data_type: Union[DataType, str]):
        return _parse_data_type(data_type)



from loguru import logger as log
from typing import Union, List
from pydantic import BaseModel

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from .base import DataFileLoader, DataFileExtractor


class ExcelDataFileLoader(DataFileLoader):

    def _convert_config_data_to_python(self,
                                       config_data: Union[str, bytes],
                                       sheet_name: Union[str, int] = 0,
                                       **kwargs) -> Union[dict, tuple, list]:
        try:
            import pandas
            return pandas.read_excel(config_data, sheet_name=sheet_name).to_dict('records')
        except ImportError as e:
            log.warning('未安装pandas模块, 无法解析excel格式数据')
            raise e

    def _read_normal_file_data(self, file_path: str, encoding: str = 'utf-8') -> Union[str, bytes]:
        with open(file_path, 'rb') as f:
            return f.read()


class ExcelDataFileExtractor(DataFileExtractor):

    def extract(self, data: Union[dict, List[dict], BaseModel, List[BaseModel]],
                **kwargs) -> str:
        raise NotImplementedError('excel不支持仅抽取数据')

    def extract_file(self,
                     file_path: str,
                     data: Union[dict, List[dict], BaseModel, List[BaseModel]],
                     sheet_name: str = 'Sheet1',
                     engine: Literal["openpyxl", "xlsxwriter"] = 'openpyxl',
                     **kwargs) -> str:
        if isinstance(data, (dict, BaseModel)):
            data = [data]
        data = [item.model_dump(mode='json') if isinstance(data, BaseModel) else item for item in data]
        for item in data:
            for k in item:
                item[k] = self._extract_value(item[k])
        try:
            import pandas
            df = pandas.DataFrame(data)
            df.to_excel(file_path, index=False, sheet_name=sheet_name, engine=engine)
            return file_path
        except ImportError as e:
            log.warning('未安装pandas模块, 无法将数据写入为excel文件')
            raise e




from dragons96_tools.ioc.beans.factory import BeanFactory
from typing import Any


class ApplicationContext:

    def __init__(self, bean_factory: BeanFactory):
        self._bean_factory = bean_factory

    def get_bean(self, name=None, cls=None):
        """获取bean, bean不存在则返回None, name和cls不能同时为空

        Args:
            name: bean名称
            cls: 函数或类
        """
        if not name and not cls:
            raise ValueError('获取bean实例name与cls参数不能同时为空')
        if self._bean_factory:
            return self._bean_factory.get_bean(name=name, required_type=cls)
        return None

    def get_bean_by_name(self, name: str):
        """通过名称获取bean

        Args:
            name: bean name

        Returns:
            bean obj or None
        """
        return self.get_bean(name=name)

    def get_bean_by_type(self, cls: Any):
        """通过类型获取bean

        Args:
            cls: bean type

        Returns:
            bean obj or None
        """
        return self.get_bean(cls=cls)



from typing import Union, List, Any

from .base import DataFileLoader, DataFileExtractor


def _load_properties_data(config_data: str):
    def dfs_properties_key(p, keys, value):
        if len(keys) == 1:
            p[keys[0]] = value
            return
        if keys[0] not in p:
            p[keys[0]] = {}
        dfs_properties_key(p[keys[0]], keys[1:], value)

    properties = {}
    for line in config_data.split('\n'):
        if not line.strip() or line.startswith('#') or '=' not in line:
            continue
        key, value = line.split('=', 1)
        key, value = key.strip(), value.strip()
        keys = key.split(".")
        if len(keys) >= 2:
            dfs_properties_key(properties, keys, value)
        else:
            properties[key] = value

    return properties


class PropertiesDataFileLoader(DataFileLoader):

    def _convert_config_data_to_python(self,
                                       config_data: Union[str, bytes],
                                       encoding: str = 'utf-8',
                                       **kwargs) -> Union[dict, tuple, list]:
        if isinstance(config_data, bytes):
            config_data = config_data.decode(encoding=encoding)
        return _load_properties_data(config_data)


class PropertiesDataFileExtractor(DataFileExtractor):

    def _convert_data_to_str(self, data: Union[dict, List[dict]], **kwargs):
        return '\n'.join(self._parse_property(data))

    def _parse_property(self, data: Any, prefix=''):
        def parse_key(key=None, index=None, prefix=''):
            if index is not None:
                if prefix:
                    return '{}[{}]'.format(prefix, index)
                return '[{}]'.format(index)
            if key and prefix:
                return '{}.{}'.format(prefix, key)
            return key or ''

        ans = []
        if isinstance(data, dict):
            for key, value in data.items():
                ans = [*ans, *(self._parse_property(value, prefix=parse_key(key, None, prefix)))]
            return ans
        if isinstance(data, list):
            for index, item in enumerate(data):
                ans = [*ans, *(self._parse_property(item, prefix=parse_key(None, index, prefix)))]
            return ans
        return ['{}={}'.format(prefix, self._extract_value(data))]



from dragons96_tools.models import BaseModel
from impala.dbapi import connect
from typing import Optional, List, Tuple, Any, Union, Type


class _ImpylaConfig(BaseModel):
    host: Optional[str] = 'localhost'
    port: Optional[int] = 21050
    user: Optional[str] = None
    password: Optional[str] = None
    db: Optional[str] = None


class ImpylaClient:

    def __init__(self,
                 host='localhost',
                 port=21050,
                 user: Optional[str] = None,
                 password: Optional[str] = None,
                 database: Optional[str] = None,
                 **kwargs):
        self._config = _ImpylaConfig(
            host=host,
            port=port,
            user=user,
            password=password,
            db=database,
        )
        self._kw = kwargs

    def _new_connection(self):
        return connect(
            host=self._config.host,
            port=self._config.port,
            user=self._config.user,
            password=self._config.password,
            database=self._config.db,
            **self._kw
        )

    def execute(self,
                sql: str,
                args: Union[List[Any], Tuple[Any], None] = None,
                modelclass: Optional[Type[BaseModel]] = None) -> Union[
        List[dict], Tuple[dict], List[BaseModel], Tuple[BaseModel]]:
        conn = self._new_connection()
        cursor = conn.cursor(dictify=True)
        try:
            cursor.execute(sql, parameters=args)
            ans = cursor.fetchall()
            if ans and modelclass:
                ans = list(map(lambda e: modelclass(**e), ans))
            return ans
        finally:
            if cursor:
                cursor.close()
            if conn:
                conn.close()



from typing import Union, List

from .base import DataFileLoader, DataFileExtractor


class IniDataFileLoader(DataFileLoader):

    def _convert_config_data_to_python(self,
                                       config_data: Union[str, bytes],
                                       encoding: str = 'utf-8',
                                       **kwargs) -> Union[dict, tuple, list]:
        if isinstance(config_data, bytes):
            config_data = config_data.decode(encoding=encoding)
        import configparser
        config = configparser.ConfigParser()
        config.read_string(config_data)
        config_dict = {section: dict(config.items(section))
                       for section in config.sections()}
        return config_dict


class IniDataFileExtractor(DataFileExtractor):

    def _convert_data_to_str(self, data: Union[dict, List[dict]], **kwargs) -> str:
        raise NotImplementedError('不支持当前类型文件提取')




import math
from typing import Union, List


def cos_similarity(a: Union[float, List[float]], b: Union[float, List[float]], normalized: bool = False) -> float:
    """余弦相似度计算, 支持皮尔逊相关系数

    Args:
        a: 计算的元素或列表
        b: 计算的元素或列表
        normalized: 是否标准化处理, Ture=是, False=否, 标准化处理变为皮尔逊相关系数
    """
    if isinstance(a, float) and isinstance(b, float):
        if a * b == 0:
            return 0
        return a * b / (abs(a) * abs(b))
    elif isinstance(a, list) and isinstance(b, list):
        if len(a) == len(b):
            top, down_a, down_b = 0, 0, 0
            normalized_avg_a = (sum(a) / len(a)) if normalized else 0
            normalized_avg_b = (sum(b) / len(b)) if normalized else 0
            for i in range(len(a)):
                top += (a[i] - normalized_avg_a) * (b[i] - normalized_avg_b)
                down_a += (a[i] - normalized_avg_a) ** 2
                down_b += (b[i] - normalized_avg_b) ** 2
            if top == 0 or down_a == 0 or down_b == 0:
                return 0
            return top / (math.sqrt(down_a) * math.sqrt(down_b))
    raise ValueError('格式不匹配无法进行相似度计算')



from .md5 import Md5DigitEnum, md5
from .base64 import encode_base16, encode_base32, encode_base64, encode_base85, \
    decode_base16, decode_base32, decode_base64, decode_base85
from .hmac import md5_hmac, sha1_hmac, sha256_hmac, sha224_hmac, sha384_hmac, sha512_hmac

import sys
if sys.version_info >= (3, 10):
    from .base64 import encode_base32hex, decode_base32hex



from typing import Any, Type, Dict, List
from .type_utils import is_basic_type
from pydantic import BaseModel
import inspect


def convert(value: Any, _type: Type) -> Any:
    """将对象转为特定类型"""
    if value is None:
        return None
    if isinstance(value, _type):
        return value
    # 基本类型直接强转
    if is_basic_type(_type):
        return _type(value)
    if isinstance(value, dict):
        if issubclass(_type, BaseModel) or getattr(_type, '__dataclass_fields__', None) is not None:
            return _type(**value)
        return _new_model(value, _type)


def _new_model(data: Dict, model_class) -> Any:
    """针对非pydantic, 使用反射"""
    members = inspect.getmembers(model_class)
    class_args_annotations_map = {}
    init_args = []
    init_args_annotations_map = {}
    for member in members:
        if member[0] == '__annotations__':
            class_args_annotations_map = member[1]
        if member[0] == '__init__':
            init_spec = inspect.getfullargspec(member[1])
            init_args = init_spec.args[1:]  # 过滤self
            init_args_annotations_map = init_spec.annotations
    # 通过构造器创建对象
    obj = _new_model_by_init_args(data, model_class, init_args, init_args_annotations_map)
    init_args_set = set(init_args)
    for member_arg, _type in class_args_annotations_map.items():
        # 构造器初始化过的参数不处理
        if member_arg in init_args_set:
            continue
        v = data.get(member_arg)
        if v is not None:
            if _type:
                setattr(obj, member_arg, convert(v, _type))
                continue
            setattr(obj, member_arg, v)
            continue
    return obj


def _new_model_by_init_args(data: Dict, model_class, init_args: List, init_args_annotations_map: Dict):
    # 先处理构造器
    if not init_args:
        obj = model_class()
    else:
        init_params = {}
        for init_arg in init_args:
            v = data.get(init_arg)
            if v is not None:
                _type = init_args_annotations_map.get(init_arg)
                if _type:
                    init_params[init_arg] = convert(v, _type)
                    continue
                init_params[init_arg] = v
                continue
            init_params[init_arg] = None
        obj = model_class(**init_params)
    return obj
